from sqlmodel import Field, SQLModel, MetaData, Relationship

from .metadata import gisaf_admin

class UserRoleLink(SQLModel, table=True):
    metadata = gisaf_admin
    __tablename__: str = 'roles_users'
    user_id: int | None = Field(
        default=None, foreign_key="user.id", primary_key=True
    )
    role_id: int | None = Field(
        default=None, foreign_key="role.id", primary_key=True
    )


class UserBase(SQLModel):
    username: str
    email: str


class User(UserBase, table=True):
    metadata = gisaf_admin
    id: int | None = Field(default=None, primary_key=True)
    roles: list["Role"] = Relationship(back_populates="users",
                                       link_model=UserRoleLink)
    password: str | None = None


class RoleBase(SQLModel):
    name: str = Field(unique=True)

class RoleWithDescription(RoleBase):
    description: str | None

class Role(RoleWithDescription, table=True):
    metadata = gisaf_admin
    id: int | None = Field(default=None, primary_key=True)
    users: list[User] = Relationship(back_populates="roles",
                                     link_model=UserRoleLink)


class UserReadNoRoles(UserBase):
    id: int
    email: str | None


class RoleRead(RoleBase):
    id: int
    users: list[UserReadNoRoles] = []


class RoleReadNoUsers(RoleBase):
    id: int


class UserRead(UserBase):
    id: int
    email: str | None
    roles: list[RoleReadNoUsers] = []