from sqlmodel import Field, SQLModel, MetaData, JSON, TEXT, Relationship, Column

from .models_base import Model
from .models_base import GeoPointMModel, BaseSurveyModel
from .project import Project
from .category import Category
from .metadata import gisaf_survey

class RawSurveyModel(BaseSurveyModel, GeoPointMModel):
    metadata = gisaf_survey
    __tablename__ = 'raw_survey'

    id: int = Field(default=None, primary_key=True)
    project_id = db.Column(db.Integer, db.ForeignKey(Project.id))
    category = db.Column(db.String, db.ForeignKey(Category.name))
    in_menu = False

    @classmethod
    def dyn_join_with(cls):
        return {
            'project': Project.on(cls.project_id == Project.id),
            'category_info': Category.on(cls.category == Category.name),
        }

    #id = db.Column(db.BigInteger, primary_key=True)
    ## XXX: Can remove the rest since it's is in the GeoPointSurveyModel class?
    #geom = db.Column(Geometry('POINTZ', srid=conf.raw_survey_srid))
    #date = db.Column(db.Date)
    #orig_id = db.Column(db.String)
    #status = db.Column(db.String(1))

    def __str__(self):
        return 'Raw Survey point id {:d}'.format(self.id)

    def to_row(self):
        """
        Get a list of attributes, typically used for exporting in CSV
        :return: list of attributes
        """
        return [
            self.id,
            self.easting,
            self.northing,
            self.elevation,
            self.category,
            self.surveyor,
            self.equipment,
            self.date.isoformat(),
            self.accuracy.name,
            self.category_info.status,
            self.project.name,
            self.orig_id
        ]

    def auto_import(self, session, model=None, status=None):
        """
        Automatically feed the raw_geom get_raw_survey_model_mapping
        :return:
        """
        if model is None:
            # XXX: move as module import?
            from gisaf.registry import registry
            model = registry.get_raw_survey_model_mapping().get(self.category)

        new_point = model(
            id=self.id,
            geom=self.geom,
            date=self.date,
            project_id=self.project_id,
            equip_id=self.equip_id,
            srvyr_id=self.srvyr_id,
            accur_id=self.accur_id,
            orig_id=self.orig_id,
            status=status,
        )
        session.merge(new_point)


class OriginRawPoint(Model):
    """
    Store information of the raw survey point used in the line work for each line and polygon shape
    Filled when importing shapefiles
    """
    __tablename__ = 'origin_raw_point'
    __table_args__ = {'schema' : 'gisaf_survey'}

    id = db.Column(db.Integer, primary_key=True)
    shape_table = db.Column(db.String, index=True)
    shape_id = db.Column(db.Integer, index=True)
    raw_point_id = db.Column(db.BigInteger)

    def __repr__(self):
        return '<models.OriginRawPoint {self.id:d} {self.shape_table:s} {self.shape_id:d} {self.raw_point_id:d}>'.format(self=self)