from typing import Any
from sqlalchemy import BigInteger
from sqlalchemy.ext.mutable import MutableDict
from sqlalchemy.dialects.postgresql import HSTORE
from sqlmodel import Field, SQLModel, MetaData, JSON, TEXT, Relationship, Column
from pydantic import computed_field

from .metadata import gisaf
from .geo_models_base import GeoPointModel

class Tags(GeoPointModel, table=True):
    metadata = gisaf
    hidden: bool = True

    class Admin:
        menu = 'Other'
        flask_admin_model_view = 'TagModelView'

    id: int | None = Field(primary_key=True)
    store: str = Field(index=True)
    ref_id: int = Field(index=True, sa_column=Column(BigInteger))
    tags: dict = Field(sa_column=Column(MutableDict.as_mutable(HSTORE)))

    def __str__(self):
        return '{self.store:s} {self.ref_id}: {self.tags}'.format(self=self)

    def __repr__(self):
        return '<models.Tag {self.store:s} {self.ref_id}: {self.tags}>'.format(self=self)


class TagKey(SQLModel, table=True):
    metadata = gisaf
    ## CREATE TABLE gisaf.tagkey (key VARCHAR(255) primary key);

    class Admin:
        menu = 'Other'
        flask_admin_model_view = 'TagKeyModelView'

    id: str | None = Field(primary_key=True)

    def __str__(self):
        return self.key

    def __repr__(self):
        return '<models.TagKey {self.key}>'.format(self=self)